@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-# Copyright (C) 2012-2014, Luis Pedro Coelho <luis@luispedro.org>
+# Copyright (C) 2012-2016, Luis Pedro Coelho <luis@luispedro.org>
 # vim: set ts=4 sts=4 sw=4 expandtab smartindent:
 #
 # License: MIT (see COPYING.MIT file)
@@ -27,7 +26,7 @@ def _as_grey(im, as_grey):
     transform = np.array([ 0.30,  0.59,  0.11])
     return np.dot(im, transform)
 
-def imread(filename, as_grey=False, formatstr=None, return_metadata=False):
+def imread(filename, as_grey=False, formatstr=None, return_metadata=False, opts=None):
     '''
     im = imread(filename, as_grey=False, formatstr={filename extension}, return_metadata=False)
     im,meta = imread(filename, as_grey=False, formatstr={filename extension}, return_metadata=True)
@@ -46,6 +45,11 @@ def imread(filename, as_grey=False, formatstr=None, return_metadata=False):
         does not correspond to the format, you can pass it explicitly.
     return_metadata : bool, optional
         Whether to return metadata (default: False)
+    opts : dict, optional
+        Other options. Support will depend on the backend. Currently used
+
+            strip_alpha: bool
+                Whether to strip the alpha channel.
 
     Returns
     -------
@@ -61,10 +65,11 @@ def imread(filename, as_grey=False, formatstr=None, return_metadata=False):
     imread_from_blob : function
         Read from a in-memory string
     '''
+    if opts is None:
+        opts = {}
     formatstr = _parse_formatstr(filename, formatstr, 'imread')
     reader = special.get(formatstr, _imread.imread)
-    flags = ('m' if return_metadata else '')
-    imdata,meta = reader(filename, formatstr, flags)
+    imdata,meta = reader(filename, formatstr, opts)
     imdata = _as_grey(imdata, as_grey)
     if return_metadata:
         return imdata, meta
@@ -73,7 +78,7 @@ def imread(filename, as_grey=False, formatstr=None, return_metadata=False):
 imload = imread
 
 
-def imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata=False):
+def imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata=False, opts=None):
     '''
     imdata = imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata={True})
     imdata,metadata = imread_from_blob(blob, formatstr=None, as_grey={False}, return_metadata=True)
@@ -95,6 +100,11 @@ def imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata=False)
         Whether to convert to grey scale image (default: no)
     return_metadata : bool, optional
         Whether to return metadata (default: False)
+    opts : dict, optional
+        Other options. Support will depend on the backend. Currently used
+
+            strip_alpha: bool
+                Whether to strip the alpha channel.
 
     Returns
     -------
@@ -110,11 +120,12 @@ def imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata=False)
     imread : function
         Read from a file on disk
     '''
+    if opts is None:
+        opts = {}
     reader = _imread.imread_from_blob
     if formatstr is None:
         formatstr = detect_format(blob, is_blob=True)
-    flags = ('m' if return_metadata else '')
-    imdata,meta = reader(blob, formatstr, flags)
+    imdata,meta = reader(blob, formatstr, opts)
     imdata = _as_grey(imdata, as_grey)
     if return_metadata:
         return imdata,meta
@@ -179,6 +190,12 @@ def imsave(filename, array, formatstr=None, metadata=None, opts=None):
             compressing 16 bit images without horizontal prediction is often
             counter-productive (see http://www.asmail.be/msg0055176395.html)
 
+
+        png:compression_level
+            Compression level to use, from 0 (no compression) to 9. Setting
+            this parameter to 0 is discouraged as setting it to 1 already
+            provides a modicum of compression at no extra computational cost.
+
     '''
     if opts is None:
         opts = {}
@@ -194,7 +211,61 @@ def imsave(filename, array, formatstr=None, metadata=None, opts=None):
         opts['metadata'] = metadata
     _imread.imsave(filename, formatstr, array, opts)
 
+def imsave_multi(filename, arrays, formatstr=None, opts=None):
+    '''Saves multiple arrays into a single file
+
+    Only certain formats (in the current version of imread, only TIFF) support
+    writing multiple outputs.
+
+    Parameters
+    ----------
+    filename : str
+        path on file system
+    arrays : list of ndarray-like
+    formatstr: str, optional
+        format string
+    metadata: bytes, optional
+        metadata to write to file. Note that not all formats support writing
+        metadata.
+    opts: dict, optional
+        This is a dictionary of options. Any non-applicable option is typically
+        silently ignored. Currently, the following options are accepted:
+
+        jpeg:quality
+            An integer 1-100 determining the quality used by JPEG backend
+            (default is libjpeg default: 75).
+
+        tiff:compress
+            Whether to use compression when saving TIFF (default: True)
+
+        tiff:horizontal-predictor
+            Whether to use horizontal prediction in TIFF. This defaults to True
+            for 16 bit images, and to False for 8 bit images. This is because
+            compressing 16 bit images without horizontal prediction is often
+            counter-productive (see http://www.asmail.be/msg0055176395.html)
+
+
+        png:compression_level
+            Compression level to use, from 0 (no compression) to 9. Setting
+            this parameter to 0 is discouraged as setting it to 1 already
+            provides a modicum of compression at no extra computational cost.
+    '''
+    if opts is None:
+        opts = {}
+    for array in arrays:
+        if not np.issubdtype(array.dtype, np.integer):
+            raise TypeError('imread.imsave: only integer images are supported')
+    arrays = [np.ascontiguousarray(array) for array in arrays]
+    if formatstr is None:
+        dot = filename.rfind('.')
+        if dot < 0:
+            raise ValueError('imread.imsave_multi: dot not found in filename ({})'.format(filename))
+        formatstr = filename[dot+1:]
+    _imread.imsave_multi(filename, formatstr, arrays, opts)
+
 imwrite = imsave
+imwrite_multi = imsave_multi
+
 
 def detect_format(filename_or_blob, is_blob=False):
     '''
@@ -216,3 +287,18 @@ def detect_format(filename_or_blob, is_blob=False):
         is returned.
     '''
     return _imread.detect_format(filename_or_blob, bool(is_blob))
+
+
+def supports_format(formatstr):
+    '''Query whether the format is supported by imread
+
+    Parameters
+    ----------
+    formatstr : str
+        format string (for example, 'png')
+
+    Returns
+    -------
+    is_supported : bool
+    '''
+    return _imread.supports_format(formatstr)
