@@ -1,7 +1,7 @@
 from nose.tools import raises
 from imread import imread
-
+from . import file_path
 
 @raises(RuntimeError)
 def test_error():
-    imread('imread/tests/data/error.unknown')
+    imread(file_path('error.unknown'))
